<?php
/**
 * pragmaMx - Usermap Module
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 1.1 $
 * $Author: tora60 $
 * $Date: 2014-03-24 16:49:34 $
 *
 * Dieses Modul basiert auf:
 *              Usermap, Copyright (C) 2004 by
 *               Andreas Trösch http://www.nukemodule.de
 *               Bernd Münt http://www.bmservices.de
 * Dieses Modul basiert auf:
 *              PHPKit Usermap 1.2 von www.fletchers-corner.de (28 Oktober 2003)
 * Plz/Geodaten von GeoNames - freie Geokoordinaten-Datenbank, http://www.geonames.org/
 */

defined('mxMainFileLoaded') or die('access denied');

/**
 * pmxUsermap
 *
 * @package
 * @author tora60
 * @copyright Copyright (c) 2014
 * @version $Id: index.php,v 1.1 2014-03-24 16:49:34 tora60 Exp $
 * @access public
 */
class pmxUsermap {
    /* the range of the map which has to be generated */
    private $x1 = 5.5; // lower longitude
    private $x2 = 17.5; // higher longitude
    private $y1 = 45.5; // lower latitude
    private $y2 = 55.3; // higher latitude
    // // altes Bild: (5.65, 15.2 , 47.2, 55.35);
    /* Abmessungen des Kartenbildes */
    private $xi = 750; // Breite Bild
    private $yi = 1024; // Höhe Bild
    private $MyMap = null;

    private $searchForUserName = false;
    private $showedusers = 0;

    /**
     * pmxUsermap::__construct()
     */
    public function __construct()
    {
        switch (true) {
            // case 'search' == $_REQUEST['op']:
            // return $this->search();
            default:
                return $this->main();
        }
    }

    /**
     * pmxUsermap::main()
     *
     * @return
     */
    private function main()
    {
        global $user_prefix;

        $module_name = basename(__DIR__);

        $mapimagename = "myusermap";
        mxGetLangfile($module_name);

        require_once(__DIR__ . '/includes/map.class.php');

        pmxHeader::add_style("modules/{$module_name}/style/style.css");

        $cacheid = $module_name;
        if (MX_IS_USER) {
            $userinfo = mxGetUserData();
            $cacheid .= serialize($userinfo['groups']);
        }

        $usernames = '';
        if (isset($_GET['usernames'])) {
            $usernames = trim($_GET['usernames']);
            $this->searchForUserName = $usernames;
            $cacheid .= $usernames;
        }
        $cacheid = md5($cacheid);
        $cache = load_class('Cache');

        if (($data = $cache->read($cacheid)) !== false) {
            $imagemap = $data['imagemap'];
            $showedusers = $data['showedusers'];
        } else {
            $this->MyMap = new Map("modules/{$module_name}/images/karte.png");

            $this->generate_map();

            $imagebin = $this->MyMap->dumpbuffer();
            $imagemap = $this->MyMap->getImageMap($mapimagename);
            $showedusers = $this->showedusers;

            $cache->write(compact('imagebin', 'imagemap', 'showedusers'), $cacheid);
        }
        unset($cache, $data, $imagebin);

        $mapimage = "modules.php?name={$module_name}&amp;file=karte&amp;id={$cacheid}";
        // $mapimage = "modules/{$module_name}/images/karte.png";
        $template = load_class('Template');
        $template->init_path(__FILE__);

        $template->assign(compact('usernames', 'module_name', 'sec', 'imagemap', 'mapimage', 'mapimagename', 'showedusers'));

        include("header.php");
        opentable();
        $template->display('main.html');
        closetable();
        include("footer.php");
    }

    private function getorte()
    {
        global $user_prefix;

        $orte = array();

        $result = sql_query("SELECT uid, uname, user_from, user_sexus
          FROM {$user_prefix}_users
          WHERE user_stat = 1
            AND user_from <> ''
            AND LENGTH(user_from) >= 5");
        while ($user = sql_fetch_assoc($result)) {
            /* 
             - bis zu 3 Buchstaben am Anfang, oder keine [1] = land
             - dann optional ein beliebiges Trennzeichen
             - anschl. beliebige Anzahl von Zahlen [2] = plz
             */

            if (!preg_match('#^([[:alpha:]]{1,3})?\W*([0-9]+)#u', $user['user_from'], $matches)) {
                /* nichts verwertbares gefunden, abbrechen und Schleife fortsetzen */
                continue;
            }


            if (!$matches[2]) {
                /* keine PLZ gefunden, abbrechen und Schleife fortsetzen */
                continue;
            }

            $land = '';
            $plz = $matches[2];

            if (!$matches[1] && (strlen($plz) == 5)) {
                /* kein Land gefunden, Länge der PLZ ist aber fünf, dann Standard Deutschland */
                $land = 'DE';
            } else {

                /* Mögliche Schreibweisen der Länder */
                $landcodes = array(
                  'AT' => array('A','AT','Ö','ÖS'),
                  'CH' => array('CH','C','SW'),
                  'DE' => array('D','DE','DT','DEU','GER') // DE als Standard am Schluss
                );

                $key = strtoupper($matches[1]);
                foreach ($landcodes as $xland => $codes) {
                    if ($key == $xland || in_array($key, $codes)) {
                        /* Wenn Treffer in der Schreibweise, diesen Wert als Land verwenden */
                        $land = $xland;
                        break;
                    }
                }
            }
            
#mxDebugFuncVars($matches[1] . ' - ' . $land . ' - ' . $plz . ' - ' .  strlen($plz));
            if (!$land) {
                /* kein passendes Land gefunden, abbrechen und Schleife fortsetzen */
                continue;
            }

            /* Koordinaten zu dieser PLZ abrufen */            
            $cord = $this->get_coordinates($land, $plz);
            if ($cord === false) {
                continue;
            }
            $key = "$cord[x]:$cord[y]";
            if (!isset($orte[$key])) {
                $orte[$key] = $cord; // gibt x und y zurück
                $orte[$key]['Land'] = $land;
                $orte[$key]['plz'] = $plz;
            }
            $orte[$key]['users'][$user['uid']] = $user;
            $this->showedusers++;
        }
        return $orte;
    }

    /**
     * pmxUsermap::generate_map()
     *
     * @param mixed $orte
     * @return
     */
    private function generate_map()
    {
        $module_name = basename(__DIR__);

        $orte = $this->getorte();
        // $Handler['onclick'] = "(navigator.appName=='Netscape')? makeVisible(event) : makeVisible();";
        $Handler['onclick'] = "makeVisible(event);";
        $Handler['onmouseout'] = ''; #"self.status='';";
        $Handler['onmouseover'] = ''; #"self.status='';return true";

        $search_usernames = array();
        if ($this->searchForUserName) {
            $search_usernames = preg_split('#\s*[.,]\s*#', strtolower($this->searchForUserName));
        }

        foreach($orte as $key => $ort) {
            $isSearched = false;
            $theNames = array();
            $genders = array();
            foreach($ort['users'] as $uid => $user) {
                if ($search_usernames && in_array(strtolower($user['uname']), $search_usernames)) {
                    $isSearched = true;
                }
                $theNames[$user['uid']] = $user['uname'];
                $genders[$user['user_sexus']] = $user['user_sexus'];
            }
            if ($isSearched) {
                $this->MyMap->addImg($ort["x"], $ort["y"], "modules/{$module_name}/images/kreuz.png", 21);
            }

            switch (true) {
                case count($genders) > 1:
                    $color = "rotblau";
                    break;
                case array_key_exists(2, $genders):
                    $color = "blau";
                    break;
                case array_key_exists(1, $genders):
                    $color = "rot";
                    break;
                default:
                    $color = "rotblau";
            }

            $this->MyMap->addPin($ort["x"], $ort["y"], "modules/{$module_name}/images/{$color}.png", 8, 1, implode(", ", $theNames), "modules.php?name={$module_name}&amp;file=viewusers&amp;names=" . urlencode(implode(".", $theNames)), $Handler['onclick'], $Handler['onmouseover'], $Handler['onmouseout']);
        }
    }

    /**
     * pmxUsermap::get_coordinates()
     *
     * @param mixed $land
     * @param mixed $plz
     * @return
     */
    private function get_coordinates($land, $plz)
    {
        global $prefix;

        static $cords = array();
        $key = "$land-$plz";

        if (isset($cords[$key])) {
            return $cords[$key];
        }

        $result = sql_query("SELECT laenge, breite
            FROM `{$prefix}_plz_data`
            WHERE land = '$land' AND plz = $plz");
        if (!sql_num_rows($result)) {
            $cords[$key] = false;
            return $cords[$key];
        }

        /* $xi * $yi, das sind die Abmessungen des Kartenbildes !! */
        list ($x, $y) = sql_fetch_row($result);
        $x = ($x - $this->x1) * $this->xi / ($this->x2 - $this->x1);
        $y = ($this->y2 - $y) * $this->yi / ($this->y2 - $this->y1);

        $x = (round($x / 8) * 8);
        $y = (round($y / 8) * 8);

        $cords[$key] = array('x' => $x, 'y' => $y);
        return $cords[$key];
    }
}

$tmp = new pmxUsermap();
$tmp = null;

?>